local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local talent = x.Talent
local variable = x.Variables
local action = x.Action
local dot = x.Dot
local target = x.Target
local buff = x.Buff
local cooldown = x.Cooldown
local azerite = x.Azerite

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.EnemyDetected, x.EnemyDetected, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Example flash color coding:
-- White - Default
-- Yellow - Limited Time or No Global Cooldown
-- Purple - AOE or Positional Damage
-- Blue - AOE Debuff
-- Orange - Finishing Move
-- Aqua - Spell Interrupt, Reflect or Dispel
-- Green - Self Buff or Turn Autocast On
-- Red - Emergency Mitigation Cooldowns or Turn Autocast Off
-- Pink - Optional

a.spells["flask_balance"] = {
	Type = "item",
	ID = 152639--[[Flask of Endless Fathoms]],
	Buff = 251837--[[Flask of Endless Fathoms]],
	BuffUnit = "player",
	FlashColor = "Green",
	CheckFirst = function(z)
		return s.Boss("target")
	end,
}

a.spells["potion_balance"] = {
	Type = "item",
	ID = 152559--[[Potion of Rising Death]],
	Buff = 269853--[[Potion of Rising Death]],
	BuffUnit = "player",
	NoGCD = true,
	CheckFirst = function(z)
		z.FlashColor = s.If(x.InCombat, "Yellow", "Green")
		return s.Boss("target")
	end,
	FlashID = {
		152559--[[Potion of Rising Death]],
		163222--[[Battle Potion of Intellect]],
	},
}

a.spells["solar_beam"] = {
	ID = 78675--[[Solar Beam]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Aqua",
}

a.spells["moonkin_form"] = {
	Type = "form",
	ID = 24858--[[Moonkin Form]],
	Buff = 24858--[[Moonkin Form]],
	BuffUnit = "player",
	FlashColor = "Green",
}

a.spells["solar_wrath"] = {
	ID = 190984--[[Solar Wrath]],
}

a.spells["potion,if=buff.celestial_alignment.up|buff.incarnation.up"] = {
	Type = "item",
	ID = 152559--[[Potion of Rising Death]],
	Buff = 269853--[[Potion of Rising Death]],
	BuffUnit = "player",
	NoGCD = true,
	CheckFirst = function(z)
		return s.Boss("target") and (buff.celestial_alignment.up_nogcd or buff.incarnation.up_nogcd)
	end,
	FlashID = {
		152559--[[Potion of Rising Death]],
		163222--[[Battle Potion of Intellect]],
	},
}

a.spells["berserking,if=buff.celestial_alignment.up|buff.incarnation.up"] = {
	ID = 26297--[[Berserking]],
	Check = function(z)
		return buff.celestial_alignment.up_nogcd or buff.incarnation.up_nogcd
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

--,"use_item,name=azurethos_singed_plumage,if=equipped.161377&(cooldown.incarnation.remains>30|cooldown.celestial_alignment.remains>30)"
--,"use_item,name=dread_gladiators_badge,if=equipped.161902&(cooldown.incarnation.remains>30|cooldown.celestial_alignment.remains>30)"
--,"use_items"

-- CDs
a.spells["warrior_of_elune"] = {
	ID = 202425--[[Warrior of Elune]],
	CheckFirst = function(z)
		return true
	end,
	FlashColor = "Yellow",
	NoGCD = true,
}

,"innervate,if=azerite.lively_spirit.enabled&(cooldown.incarnation.up|cooldown.celestial_alignment.remains<12)&(((raid_event.adds.duration%15)*(4)<(raid_event.adds.in%180))|(raid_event.adds.up))"

a.spells["incarnation,if=astral_power>=40&(((raid_event.adds.duration/30)*(4)<(raid_event.adds.in/180))|(raid_event.adds.up))"] = {
	ID = 102560--[[Incarnation: Chosen of Elune]],
	CheckFirst = function(z)
		return x.AstralPower >= 40 and x.UseLongCD
	end,
	FlashColor = "Pink",
}

a.spells["celestial_alignment,if=astral_power>=40&(!azerite.lively_spirit.enabled|buff.lively_spirit.up)&(((raid_event.adds.duration/15)*(4)<(raid_event.adds.in/180))|(raid_event.adds.up))"] = {
	ID = 194223--[[Celestial Alignment]],
	CheckFirst = function(z)
		return x.AstralPower >= 40 and (not azerite.lively_spirit.enabled or buff.lively_spirit.up) and x.UseLongCD
	end,
	FlashColor = "Pink",
}

a.spells["fury_of_elune,if=(((raid_event.adds.duration/8)*(4)<(raid_event.adds.in/60))|(raid_event.adds.up))&((buff.celestial_alignment.up|buff.incarnation.up)|(cooldown.celestial_alignment.remains>30|cooldown.incarnation.remains>30))"] = {
	ID = 202770--[[Fury of Elune]],
	CheckFirst = function(z)
		return x.UseLongCD and ((buff.celestial_alignment.up or buff.incarnation.up) or (cooldown.celestial_alignment.remains > 30 or cooldown.incarnation.remains>30))
	end,
	NoRangeCheck = true,
	FlashColor = "Pink",
}

a.spells["force_of_nature,if=(buff.celestial_alignment.up|buff.incarnation.up)|(cooldown.celestial_alignment.remains>30|cooldown.incarnation.remains>30)"] = {
	ID = 205636--[[Force of Nature]],
	CheckFirst = function(z)
		return x.UseLongCD and ((buff.celestial_alignment.up or buff.incarnation.up) or (cooldown.celestial_alignment.remains > 30 or cooldown.incarnation.remains>30))
	end,
	NoRangeCheck = true,
	FlashColor = "Pink",
}

-- DoTs
a.spells["sunfire,target_if=refreshable,if=astral_power.deficit>=7&target.time_to_die>5.4&(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.sunfire)&(movement.distance>0|raid_event.movement.in>remains|remains<=execute_time*2)"] = {
	ID = 93402--[[Sunfire]],
	CheckFirst = function(z)
		return dot.sunfire.refreshable and x.AstralPowerDeficit >= 7 and target.time_to_die > 5.4 and (not buff.celestial_alignment.up and not buff.incarnation.up or not variable.az_streak or not prev_gcd[1].sunfire) and (dot.moonfire.remains <= x.MaxGCD * 2)
	end,
}

a.spells["moonfire,target_if=refreshable,if=astral_power.deficit>=7&target.time_to_die>6.6&(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.moonfire)&(movement.distance>0|raid_event.movement.in>remains|remains<=execute_time*2)"] = {
	ID = 8921--[[Moonfire]],
	CheckFirst = function(z)
		return dot.moonfire.refreshable and x.AstralPowerDeficit >= 7 and target.time_to_die > 6.6 and (not buff.celestial_alignment.up and not buff.incarnation.up or not variable.az_streak or not prev_gcd[1].moonfire) and (dot.moonfire.remains <= x.MaxGCD * 2)
	end,
}

a.spells["stellar_flare,target_if=refreshable,if=astral_power.deficit>=12&target.time_to_die>7.2&(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.stellar_flare)"] = {
	ID = 202347--[[Stellar Flare]],
	NotIfActive = true,
	CheckFirst = function(z)
		return dot.stellar_flare.refreshable and x.AstralPowerDeficit >= 12 and target.time_to_die > 7.2 and (not buff.celestial_alignment.up and not buff.incarnation.up or not variable.az_streak or not prev_gcd[1].stellar_flare)
	end,
}

-- Empowerment cap check

a.spells["lunar_strike,if=astral_power.deficit>=16&(buff.lunar_empowerment.stack=3|(spell_targets<3&astral_power>=40&(buff.lunar_empowerment.stack=2&buff.solar_empowerment.stack=2)))&!(spell_targets.moonfire>=2&variable.az_potm=3&active_enemies=2)"] = {
	ID = 194153--[[Lunar Strike]],
	CheckFirst = function(z)
		return x.AstralPowerDeficit >= 16 and (buff.lunar_empowerment.stack == 3 or (x.ActiveEnemies < 3 and x.AstralPower >= 40 and (buff.lunar_empowerment.stack == 2 and buff.solar_empowerment.stack == 2))) and not (x.ActiveEnemies >= 2 and variable.az_potm == 3 and x.ActiveEnemies == 2)
	end,
}

a.spells["solar_wrath,if=astral_power.deficit>=12&(buff.solar_empowerment.stack=3|(variable.az_sb>1&spell_targets.starfall<3&astral_power>=32&!buff.sunblaze.up))&!(spell_targets.moonfire>=2&active_enemies<=4&variable.az_potm=3)|(variable.az_streak&(buff.celestial_alignment.up|buff.incarnation.up)&!prev_gcd.1.solar_wrath&astral_power.deficit>=12)"] = {
	ID = 190984--[[Solar Wrath]],
	CheckFirst = function(z)
		return x.AstralPowerDeficit >= 12 and (buff.solar_empowerment.stack == 3 or (variable.az_sb > 1 and x.ActiveEnemies < 3 and x.AstralPower >= 32 and not buff.sunblaze.up)) and not (x.ActiveEnemies >= 2 and x.ActiveEnemies <= 4 and variable.az_potm == 3) or (variable.az_streak and (buff.celestial_alignment.up or buff.incarnation.up) and not prev_gcd[1].solar_wrath and astral_power.deficit >= 12)
	end,
}

-- Rotation
a.spells["starsurge,if=(spell_targets.starfall<3&(!buff.starlord.up|buff.starlord.remains>=4)|execute_time*(astral_power/40)>target.time_to_die)&(!buff.celestial_alignment.up&!buff.incarnation.up|variable.az_streak<2|!prev_gcd.1.starsurge)&(raid_event.movement.in>(buff.lunar_empowerment.stack*action.lunar_strike.execute_time+buff.solar_empowerment.stack*action.solar_wrath.execute_time)|(astral_power+buff.lunar_empowerment.stack*12+buff.solar_empowerment.stack*8)>=96)"] = {
	ID = 78674--[[Starsurge]],
	CheckFirst = function(z)
		return (x.ActiveEnemies < 3 and (not buff.starlord.up or buff.starlord.remains >= 4) or x.MaxGCD * (x.AstralPower / 40) > target.time_to_die) and (not buff.celestial_alignment.up and not buff.incarnation.up or variable.az_streak < 2 or not prev_gcd[1].starsurge) and (x.AstralPower + buff.lunar_empowerment.stack * 12 + buff.solar_empowerment.stack * 8) >= 96
	end,
}

a.spells["starfall,if=spell_targets.starfall>=3&(!buff.starlord.up|buff.starlord.remains>=4)"] = {
	ID = 191034--[[Starfall]],
	NoRangeCheck = true,
	Check = function(z)
		return x.ActiveEnemies >= 3 and (not buff.starlord.up or buff.starlord.remains>=4)
	end,
}

a.spells["new_moon,if=astral_power.deficit>10+execute_time/1.5"] = {
	ID = 274281--[[New Moon]],
	CheckFirst = function(z)
		return x.AstralPowerDeficit > 10 + action.new_moon.execute_time / 1.5
	end,
}

a.spells["half_moon,if=astral_power.deficit>20+execute_time/1.5"] = {
	ID = 274282--[[Half Moon]],
	CheckFirst = function(z)
		return x.AstralPowerDeficit > 20 + action.half_moon.execute_time / 1.5
	end,
}

a.spells["full_moon,if=astral_power.deficit>40+execute_time/1.5"] = {
	ID = 274283--[[Full Moon]],
	CheckFirst = function(z)
		return x.AstralPowerDeficit > 40 + action.full_moon.execute_time / 1.5
	end,
}

a.spells["lunar_strike,if=((buff.warrior_of_elune.up|buff.lunar_empowerment.up|spell_targets>=3&!buff.solar_empowerment.up)&(!buff.celestial_alignment.up&!buff.incarnation.up|variable.az_streak<2|!prev_gcd.1.lunar_strike)|(variable.az_ds&!buff.dawning_sun.up))&!(spell_targets.moonfire>=2&active_enemies<=4&(variable.az_potm=3|variable.az_potm=2&active_enemies=2))"] = {
	ID = 194153--[[Lunar Strike]],
	CheckFirst = function(z)
		return ((buff.warrior_of_elune.up or buff.lunar_empowerment.up or x.ActiveEnemies >= 3 and not buff.solar_empowerment.up) and (not buff.celestial_alignment.up and not buff.incarnation.up or variable.az_streak < 2 or not prev_gcd[1].lunar_strike) or (variable.az_ds and not buff.dawning_sun.up)) and not (x.ActiveEnemies >= 2 and x.ActiveEnemies <= 4 and (variable.az_potm  == 3 or variable.az_potm == 2 and x.ActiveEnemies == 2))
	end,
}

a.spells["solar_wrath,if=(!buff.celestial_alignment.up&!buff.incarnation.up|variable.az_streak<2|!prev_gcd.1.solar_wrath)&!(spell_targets.moonfire>=2&active_enemies<=4&(variable.az_potm=3|variable.az_potm=2&active_enemies=2))"] = {
	ID = 190984--[[Solar Wrath]],
	CheckFirst = function(z)
		return (not buff.celestial_alignment.up and not buff.incarnation.up or variable.az_streak < 2 or not prev_gcd[1].solar_wrath) and not (x.ActiveEnemies >= 2 and x.ActiveEnemies <= 4 and (variable.az_potm ==3 or variable.az_potm ==2 and x.ActiveEnemies == 2))
	end,
}

a.spells["sunfire,if=(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.sunfire)&!(variable.az_potm>=2&spell_targets.moonfire>=2)"] = {
	ID = 93402--[[Sunfire]],
	CheckFirst = function(z)
		return (not buff.celestial_alignment.up and not buff.incarnation.up or not variable.az_streak or not prev_gcd[1].sunfire) and not (variable.az_potm >= 2 and x.ActiveEnemies >= 2)
	end,
}

a.spells["moonfire"] = {
	ID = 8921--[[Moonfire]],
	CheckFirst = function(z)
		return true
	end,
}
