local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local talent = x.Talent
local variable = x.Variables
local dot = x.Dot
local buff = x.Buff
local azerite = x.Azerite
local target = x.Target

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.EnemyDetected, x.EnemyDetected, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Example flash color coding:
-- White - Default
-- Yellow - Limited Time or No Global Cooldown
-- Purple - AOE or Positional Damage
-- Blue - AOE Debuff
-- Orange - Finishing Move
-- Aqua - Spell Interrupt, Reflect or Dispel
-- Green - Self Buff or Turn Autocast On
-- Red - Emergency Mitigation Cooldowns or Turn Autocast Off
-- Pink - Optional

a.spells["flask_feral"] = {
	Type = "item",
	ID = 152638--[[Flask of the Currents]],
	Buff = 251836--[[Flask of the Currents]],
	BuffUnit = "player",
	FlashColor = "Green",
	CheckFirst = function(z)
		return s.Boss("target")
	end,
}

a.spells["potion_feral"] = {
	Type = "item",
	ID = 163223--[[Battle Potion of Agility]],
	Buff = 279152--[[Battle Potion of Agility]],
	BuffUnit = "player",
	NoGCD = true,
	CheckFirst = function(z)
		z.FlashColor = s.If(x.InCombat, "Yellow", "Green")
		return s.Boss("target")
	end,
}

a.spells["wild_charge_cat"] = {
	ID = 49376--[[Wild Charge]],
	EnemyTargetNeeded = true,
	CheckFirst = function(z)
		return s.SpellInRange(49376--[[Wild Charge]], "target")
	end,
	FlashColor = "Pink",
}

a.spells["regrowth,if=talent.bloodtalons.enabled"] = {
	ID = 8936--[[Regrowth]],
	Buff = 8936--[[Regrowth]],
	BuffUnit = "player",
	NotWhileActive = true,
	NoRangeCheck = true,
	CheckFirst = function(z)
		return talent.bloodtalons.enabled
	end,
}

a.spells["cat_form"] = {
	Type = "form",
	ID = 768--[[Cat Form]],
	Buff = 768--[[Cat Form]],
	BuffUnit = "player",
	FlashColor = "Green",
	CheckFirst = function(z)
		return x.ShouldPermanentBuff
	end,
}

a.spells["prowl"] = {
	ID = 5215--[[Prowl]],
	Buff = 5215--[[Prowl]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return not x.InCombat and x.ShouldTemporaryBuff
	end,
	NoGCD = true,
	FlashColor = "Green",
}

a.spells["berserk"] = {
	ID = 106951--[[Berserk]],
	CheckFirst = function(z)
		return x.UseLongCD
	end,
	FlashColor = "Pink",
}

a.spells["tigers_fury,if=variable.delayed_tf_opener=0"] = {
	ID = 5217--[[Tiger's Fury]],
	Buff = 5217--[[Tiger's Fury]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return variable.delayed_tf_opener == 0
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["rake,if=!ticking|buff.prowl.up"] = {
	ID = 1822--[[Rake]],
	CheckFirst = function(z)
		return not dot.rake.ticking or buff.prowl.up
	end,
}

a.spells["moonfire_cat,if=!ticking|buff.bloodtalons.stack=1&combo_points<5"] = {
	ID = 155625--[[Moonfire]],
	CheckFirst = function(z)
		return talent.lunar_inspiration.enabled and (not dot.moonfire_cat.ticking or buff.bloodtalons.stack == 1 and x.CP < 5)
	end,
}

a.spells["thrash,if=!ticking&combo_points<5"] = {
	ID = 106830--[[Thrash]],
	NoRangeCheck = true,
	Melee = true,
	CheckFirst = function(z)
		return not dot.thrash.ticking and x.CP < 5
	end,
}

a.spells["shred,if=combo_points<5"] = {
	ID = 5221--[[Shred]],
	CheckFirst = function(z)
		return x.CP < 5
	end,
}

a.spells["regrowth,if=combo_points=5&talent.bloodtalons.enabled&(talent.sabertooth.enabled&buff.bloodtalons.down|buff.predatory_swiftness.up)"] = {
	ID = 8936--[[Regrowth]],
	Buff = 8936--[[Regrowth]],
	BuffUnit = "player",
	NotWhileActive = true,
	NoRangeCheck = true,
	CheckFirst = function(z)
		return x.CP == 5 and talent.bloodtalons.enabled and (talent.sabertooth.enabled and buff.bloodtalons.down or buff.predatory_swiftness.up)
	end,
}

a.spells["tigers_fury"] = {
	ID = 5217--[[Tiger's Fury]],
	Buff = 5217--[[Tiger's Fury]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return true
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["rip,if=combo_points=5"] = {
	ID = 1079--[[Rip]],
	CheckFirst = function(z)
		return x.CP == 5
	end,
}

a.spells["cat_form,if=!buff.cat_form.up"] = {
	Type = "form",
	ID = 768--[[Cat Form]],
	Buff = 768--[[Cat Form]],
	BuffUnit = "player",
}

a.spells["rake,if=buff.prowl.up|buff.shadowmeld.up"] = {
	ID = 1822--[[Rake]],
	CheckFirst = function(z)
		return buff.prowl.up
	end,
}

a.spells["dash,if=!buff.cat_form.up"] = {
	ID = 1850--[[Dash]],
	Buff = 768--[[Cat Form]],
	BuffUnit = "player",
	FlashColor = "Pink",
}

a.spells["prowl,if=buff.incarnation.remains<0.5&buff.jungle_stalker.up"] = {
	ID = 5215--[[Prowl]],
	Buff = 5215--[[Prowl]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return buff.incarnation.remains < 1.5 and buff.jungle_stalker.up
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["berserk,if=energy>=30&(cooldown.tigers_fury.remains>5|buff.tigers_fury.up)"] = {
	ID = 106951--[[Berserk]],
	CheckFirst = function(z)
		return x.UseLongCD and (x.Energy >= 30 and (cooldown.tigers_fury.remains > 5 or buff.tigers_fury.up))
	end,
	FlashColor = "Pink",
}

a.spells["tigers_fury,if=energy.deficit>=60"] = {
	ID = 5217--[[Tiger's Fury]],
	Buff = 5217--[[Tiger's Fury]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return x.MaxEnergy - x.Energy >= 60
	end,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["berserking"] = {
	ID = 26297--[[Berserking]],
	FlashColor = "Yellow",
	NoGCD = true,
}

a.spells["feral_frenzy,if=combo_points=0"] = {
	ID = 274837--[[Feral Frenzy]],
	CheckFirst = function(z)
		return x.CP == 0
	end,
}

a.spells["incarnation,if=energy>=30&(cooldown.tigers_fury.remains>15|buff.tigers_fury.up)"] = {
	ID = 102543--[[Incarnation: King of the Jungle]],
	CheckFirst = function(z)
		return x.UseLongCD and (x.Energy >= 30 and (cooldown.tigers_fury.remains > 15 or buff.tigers_fury.up))
	end,
	FlashColor = "Pink",
}

a.spells["potion,name=battle_potion_of_agility,if=target.time_to_die<65|(time_to_die<180&(buff.berserk.up|buff.incarnation.up))"] = {
	Type = "item",
	ID = 163223--[[Battle Potion of Agility]],
	Buff = 279152--[[Battle Potion of Agility]],
	BuffUnit = "player",
	NoGCD = true,
	FlashColor = "Yellow",
	CheckFirst = function(z)
		return s.Boss("target") and (target.time_to_die < 65 or (target.time_to_die < 180 and (buff.berserk.up_nogcd or buff.incarnation.up_nogcd)))
	end,
}

,"shadowmeld,if=combo_points<5&energy>=action.rake.cost&dot.rake.pmultiplier<2.1&buff.tigers_fury.up&(buff.bloodtalons.up|!talent.bloodtalons.enabled)&(!talent.incarnation.enabled|cooldown.incarnation.remains>18)&!buff.incarnation.up"

a.spells["ferocious_bite,target_if=dot.rip.ticking&dot.rip.remains<3&target.time_to_die>10&(target.health.pct<25|talent.sabertooth.enabled)"] = {
	ID = 22568--[[Ferocious Bite]],
	CheckFirst = function(z)
		return x.CP > 0 and dot.rip.ticking and dot.rip.remains < 3 and target.time_to_die > 10 and (target.health.pct < 25 or talent.sabertooth.enabled)
	end,
}

a.spells["regrowth,if=combo_points=5&buff.predatory_swiftness.up&talent.bloodtalons.enabled&buff.bloodtalons.down&(!buff.incarnation.up|dot.rip.remains<8)"] = {
	ID = 8936--[[Regrowth]],
	NotWhileActive = true,
	NoRangeCheck = true,
	CheckFirst = function(z)
		return x.CP == 5 and buff.predatory_swiftness.up and talent.bloodtalons.enabled and buff.bloodtalons.down and (not buff.incarnation.up or dot.rip.remains < 8)
	end,
}

--				,"pool_resource,for_next=1"
a.spells["savage_roar,if=buff.savage_roar.down"] = {
	ID = 52610--[[Savage Roar]],
	NoPowerCheck = true,
	NoRangeCheck = true,
	Check = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(52610--[[Savage Roar]]), 40, 100)
		return talent.savage_roar.enabled and buff.savage_roar.down and x.CP > 0
	end,
}

--				,"pool_resource,for_next=1"
a.spells["rip,target_if=!ticking|(remains<=duration*0.3)&(target.health.pct>25&!talent.sabertooth.enabled)|(remains<=duration*0.8&persistent_multiplier>dot.rip.pmultiplier)&target.time_to_die>8"] = {
	ID = 1079--[[Rip]],
	NoPowerCheck = true,
	CheckFirst = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(1079--[[Rip]]), 40, 100)
		return x.CP > 0 and (not dot.rip.ticking or dot.rip.refreshable and (target.health.pct > 25 and not talent.sabertooth.enabled) or (dot.rip.remains <= dot.rip.duration * 0.8 and false) and target.time_to_die > 8)
	end,
}

--				,"pool_resource,for_next=1"
a.spells["savage_roar,if=buff.savage_roar.remains<12"] = {
	ID = 52610--[[Savage Roar]],
	NoPowerCheck = true,
	NoRangeCheck = true,
	Check = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(52610--[[Savage Roar]]), 40, 100)
		return talent.savage_roar.enabled and buff.savage_roar.remains < 12 and x.CP > 0
	end,
}

--				,"pool_resource,for_next=1"
a.spells["maim,if=buff.iron_jaws.up"] = {
	ID = 22570--[[Maim]],
	NoPowerCheck = true,
	Check = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(22570--[[Maim]]), 40, 100)
		return buff.iron_jaws.up and x.CP > 0
	end,
}

a.spells["ferocious_bite,max_energy=1"] = {
	ID = 22568--[[Ferocious Bite]],
	CheckFirst = function(z)
		return x.CP > 0 and x.Energy == x.MaxEnergy
	end,
}

a.spells["regrowth,if=talent.bloodtalons.enabled&buff.predatory_swiftness.up&buff.bloodtalons.down&combo_points=4&dot.rake.remains<4"] = {
	ID = 8936--[[Regrowth]],
	NotWhileActive = true,
	NoRangeCheck = true,
	CheckFirst = function(z)
		return talent.bloodtalons.enabled and buff.predatory_swiftness.up and buff.bloodtalons.down and x.CP == 4 and dot.rake.remains < 4
	end,
}

a.spells["regrowth,if=talent.bloodtalons.enabled&buff.bloodtalons.down&buff.predatory_swiftness.up&talent.lunar_inspiration.enabled&dot.rake.remains<1"] = {
	ID = 8936--[[Regrowth]],
	NotWhileActive = true,
	NoRangeCheck = true,
	CheckFirst = function(z)
		return talent.bloodtalons.enabled and buff.bloodtalons.down and buff.predatory_swiftness.up and talent.lunar_inspiration.enabled and dot.rake.remains < 1
	end,
}

a.spells["brutal_slash,if=spell_targets.brutal_slash>desired_targets"] = {
	ID = 202028--[[Brutal Slash]],
	NoRangeCheck = true,
	Melee = true,
	CheckFirst = function(z)
		return x.ActiveEnemies > 2
	end,
}

--			,"pool_resource,for_next=1"
a.spells["thrash_cat,if=refreshable&(spell_targets.thrash_cat>2)"] = {
	ID = 106830--[[Thrash]],
	NoRangeCheck = true,
	NoPowerCheck = true,
	Melee = true,
	CheckFirst = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(106830--[[Thrash]]), 40, 100)
		return dot.thrash.refreshable and x.ActiveEnemies > 2
	end,
}

--			,"pool_resource,for_next=1"
a.spells["rake,target_if=!ticking|(!talent.bloodtalons.enabled&remains<duration*0.3)&target.time_to_die>4"] = {
	ID = 1822--[[Rake]],
	NoPowerCheck = true,
	CheckFirst = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(1822--[[Rake]]), 40, 100)
		return not dot.rake.ticking or (not talent.bloodtalons.enabled and dot.rake.refreshable) and target.time_to_die > 4
	end,
}

--			,"pool_resource,for_next=1"
a.spells["rake,target_if=talent.bloodtalons.enabled&buff.bloodtalons.up&((remains<=7)&persistent_multiplier>dot.rake.pmultiplier*0.85)&target.time_to_die>4"] = {
	ID = 1822--[[Rake]],
	NoPowerCheck = true,
	CheckFirst = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(1822--[[Rake]]), 40, 100)
		return talent.bloodtalons.enabled and buff.bloodtalons.up and ((dot.rake.remains <= 7) and false) and target.time_to_die > 4
	end,
}

-- With LI & BT, we can use moonfire to save BT charges, allowing us to better refresh rake
a.spells["moonfire_cat,if=buff.bloodtalons.up&buff.predatory_swiftness.down&combo_points<5"] = {
	ID = 155625--[[Moonfire]],
	CheckFirst = function(z)
		return talent.lunar_inspiration.enabled and buff.bloodtalons.up and buff.predatory_swiftness.down and x.CP < 5
	end,
}

a.spells["brutal_slash,if=(buff.tigers_fury.up&(raid_event.adds.in>(1+max_charges-charges_fractional)*recharge_time))"] = {
	ID = 202028--[[Brutal Slash]],
	NoRangeCheck = true,
	Melee = true,
	CheckFirst = function(z)
		return buff.tigers_fury.up
	end,
}

a.spells["moonfire_cat,target_if=refreshable"] = {
	ID = 155625--[[Moonfire]],
	CheckFirst = function(z)
		return talent.lunar_inspiration.enabled and dot.moonfire_cat.refreshable
	end,
}
--			,"pool_resource,for_next=1"
a.spells["thrash_cat,if=refreshable&((variable.use_thrash=2&(!buff.incarnation.up|azerite.wild_fleshrending.enabled))|spell_targets.thrash_cat>1)"] = {
	ID = 106830--[[Thrash]],
	NoRangeCheck = true,
	NoPowerCheck = true,
	Melee = true,
	CheckFirst = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(106830--[[Thrash]]), 40, 100)
		return dot.thrash.refreshable and ((variable.use_thrash == 2 and (not buff.incarnation.up or azerite.wild_fleshrending.enabled)) or x.ActiveEnemies > 1)
	end,
}

a.spells["thrash_cat,if=refreshable&variable.use_thrash=1&buff.clearcasting.react&(!buff.incarnation.up|azerite.wild_fleshrending.enabled)"] = {
	ID = 106830--[[Thrash]],
	NoRangeCheck = true,
	Melee = true,
	CheckFirst = function(z)
		return dot.thrash.refreshable and variable.use_thrash == 1 and buff.clearcasting.up and (not buff.incarnation.up or azerite.wild_fleshrending.enabled)
	end,
}

--			,"pool_resource,for_next=1"
a.spells["swipe_cat,if=spell_targets.swipe_cat>1"] = {
	ID = 106785--[[Swipe]],
	NoRangeCheck = true,
	NoPowerCheck = true,
	Melee = true,
	CheckFirst = function(z)
		z.FlashBrightness = s.If(x.Energy < s.SpellCost(106785--[[Swipe]]), 40, 100)
		return x.ActiveEnemies > 1
	end,
}

a.spells["shred,if=buff.clearcasting.react"] = {
	ID = 5221--[[Shred]],
	CheckFirst = function(z)
		return buff.clearcasting.up
	end,
}

-- With Power of the Moon traits, Moonfire becomes our primary builder outside of Clearcasting and Incarnation
a.spells["moonfire_cat,if=azerite.power_of_the_moon.enabled&!buff.incarnation.up"] = {
	ID = 155625--[[Moonfire]],
	CheckFirst = function(z)
		return talent.lunar_inspiration.enabled and azerite.power_of_the_moon.enabled and not buff.incarnation.up
	end,
}

a.spells["shred,if=dot.rake.remains>(action.shred.cost+action.rake.cost-energy)/energy.regen|buff.clearcasting.react"] = {
	ID = 5221--[[Shred]],
	CheckFirst = function(z)
		return dot.rake.remains > (s.SpellCost(5221--[[Shred]]) + s.SpellCost(1822--[[Rake]]) - x.Energy) / s.PowerRegeneration()
	end,
}
