local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local equipped = x.Equipped
local dot = x.Dot
local variable = x.Variables
local azerite = x.Azerite
local talent = x.Talent
local buff = x.Buff

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff


-- Use this single spam function and remove the multiple spam table below, or just use the multiple spam table below and leave this function in place.
a.SetSpamFunction(function() a.RunSpamTable()

	s.UpdateTarget()

	x.InCombat = s.InCombat()
	x.Spec = s.TalentMastery()
	x.MaxHealth = s.MaxHealth("player")
	x.Health = s.Health("player")
	x.ActiveEnemies = s.SpellTargets()
	x.MaxGCD = math.max(1, 150 / (100 + UnitSpellHaste("player")))
	x.UseLongCD = s.UseLongCD()

	if x.Spec == 1 then
		-- Balance

		x.AstralPower = s.MyModifiedPower(Enum.PowerType.AstralPower)
		x.AstralPowerDeficit = UnitPowerMax("player", Enum.PowerType.AstralPower) - x.AstralPower

		-- Azerite variables
		-- Streaking Stars:
		-- Cast sw after every non-sw spell unless capping, sw sf sw if forced
		variable.az_streak = azerite.streaking_stars.rank
		variable.az_ds = azerite.dawning_sun.rank
		variable.az_sb = azerite.sunblaze.rank
		-- Power of the Moon:
		-- 2T: 3x noemp, 2x emp, 1x normal
		-- 3T: 3x lsemp, 2x normal, 1x normal
		-- 4T: 3x lsemp, 2x normal, 1x normal
		variable.az_potm = s.If(talent.twin_moons.enabled, azerite.power_of_the_moon.rank, 0)

		-- Executed before combat begins. Accepts non-harmful actions only.
		if not x.InCombat and x.Enemy and not s.Casting(nil, "player") then
			a.Flash(
				"flask_balance"
				,"food"
				,"augmentation"
				,"moonkin_form"
				,"potion_balance"
				,"solar_wrath"
			)
			return
		end

		a.Flash("solar_beam")

		-- Executed every time the actor is available.
		s.UpdateBuff(194223, "Celestial Alignment")
		s.UpdateBuff(102560, "Incarnation")
		s.UpdateCooldown(194223, "Celestial Alignment")
		s.UpdateCooldown(102560, "Incarnation")
		a.Flash(
			"potion,if=buff.celestial_alignment.up|buff.incarnation.up"
			,"berserking,if=buff.celestial_alignment.up|buff.incarnation.up"
			--,"use_item,name=azurethos_singed_plumage,if=equipped.161377&(cooldown.incarnation.remains>30|cooldown.celestial_alignment.remains>30)"
			--,"use_item,name=dread_gladiators_badge,if=equipped.161902&(cooldown.incarnation.remains>30|cooldown.celestial_alignment.remains>30)"
			--,"use_items"
			,"warrior_of_elune"
		)

		-- CDs
		a.Flash(
--			,"innervate,if=azerite.lively_spirit.enabled&(cooldown.incarnation.up|cooldown.celestial_alignment.remains<12)&(((raid_event.adds.duration/15)*(4)<(raid_event.adds.in%180))|(raid_event.adds.up))"
			"incarnation,if=astral_power>=40&(((raid_event.adds.duration/30)*(4)<(raid_event.adds.in/180))|(raid_event.adds.up))"
			,"celestial_alignment,if=astral_power>=40&(!azerite.lively_spirit.enabled|buff.lively_spirit.up)&(((raid_event.adds.duration/15)*(4)<(raid_event.adds.in/180))|(raid_event.adds.up))"
			,"fury_of_elune,if=(((raid_event.adds.duration/8)*(4)<(raid_event.adds.in/60))|(raid_event.adds.up))&((buff.celestial_alignment.up|buff.incarnation.up)|(cooldown.celestial_alignment.remains>30|cooldown.incarnation.remains>30))"
			,"force_of_nature,if=(buff.celestial_alignment.up|buff.incarnation.up)|(cooldown.celestial_alignment.remains>30|cooldown.incarnation.remains>30)"
		)

		s.UpdateBuff(164545, "Solar Empowerment")
		s.UpdateBuff(164547, "Lunar Empowerment")
		s.UpdateBuff(279648, "Lively Spirit")
		s.UpdateBuff(279709, "Starlord")
		s.UpdateBuff(274399, "Sunblaze")
		s.UpdateBuff(202425, "Warrior of Elune")
		s.UpdateBuff(276154, "Dawning Sun")
		s.UpdateDoT(164812, "Moonfire")
		s.UpdateDoT(164815, "Sunfire")
		s.UpdateDoT(202347, "Stellar Flare")
		s.UpdateCastTime(274281, "New Moon")
		s.UpdateCastTime(274282, "Half Moon")
		s.UpdateCastTime(274283, "Full Moon")
		a.Flash(
			-- DoTs
			"sunfire,target_if=refreshable,if=astral_power.deficit>=7&target.time_to_die>5.4&(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.sunfire)&(movement.distance>0|raid_event.movement.in>remains|remains<=execute_time*2)"
			,"moonfire,target_if=refreshable,if=astral_power.deficit>=7&target.time_to_die>6.6&(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.moonfire)&(movement.distance>0|raid_event.movement.in>remains|remains<=execute_time*2)"
			,"stellar_flare,target_if=refreshable,if=astral_power.deficit>=12&target.time_to_die>7.2&(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.stellar_flare)"

			-- Empowerment cap check
			,"lunar_strike,if=astral_power.deficit>=16&(buff.lunar_empowerment.stack=3|(spell_targets<3&astral_power>=40&(buff.lunar_empowerment.stack=2&buff.solar_empowerment.stack=2)))&!(spell_targets.moonfire>=2&variable.az_potm=3&active_enemies=2)"
			,"solar_wrath,if=astral_power.deficit>=12&(buff.solar_empowerment.stack=3|(variable.az_sb>1&spell_targets.starfall<3&astral_power>=32&!buff.sunblaze.up))&!(spell_targets.moonfire>=2&active_enemies<=4&variable.az_potm=3)|(variable.az_streak&(buff.celestial_alignment.up|buff.incarnation.up)&!prev_gcd.1.solar_wrath&astral_power.deficit>=12)"

			-- Rotation
			,"starsurge,if=(spell_targets.starfall<3&(!buff.starlord.up|buff.starlord.remains>=4)|execute_time*(astral_power/40)>target.time_to_die)&(!buff.celestial_alignment.up&!buff.incarnation.up|variable.az_streak<2|!prev_gcd.1.starsurge)&(raid_event.movement.in>(buff.lunar_empowerment.stack*action.lunar_strike.execute_time+buff.solar_empowerment.stack*action.solar_wrath.execute_time)|(astral_power+buff.lunar_empowerment.stack*12+buff.solar_empowerment.stack*8)>=96)"
			,"starfall,if=spell_targets.starfall>=3&(!buff.starlord.up|buff.starlord.remains>=4)"
			,"new_moon,if=astral_power.deficit>10+execute_time/1.5"
			,"half_moon,if=astral_power.deficit>20+execute_time/1.5"
			,"full_moon,if=astral_power.deficit>40+execute_time/1.5"
			,"lunar_strike,if=((buff.warrior_of_elune.up|buff.lunar_empowerment.up|spell_targets>=3&!buff.solar_empowerment.up)&(!buff.celestial_alignment.up&!buff.incarnation.up|variable.az_streak<2|!prev_gcd.1.lunar_strike)|(variable.az_ds&!buff.dawning_sun.up))&!(spell_targets.moonfire>=2&active_enemies<=4&(variable.az_potm=3|variable.az_potm=2&active_enemies=2))"
			,"solar_wrath,if=(!buff.celestial_alignment.up&!buff.incarnation.up|variable.az_streak<2|!prev_gcd.1.solar_wrath)&!(spell_targets.moonfire>=2&active_enemies<=4&(variable.az_potm=3|variable.az_potm=2&active_enemies=2))"
			,"sunfire,if=(!buff.celestial_alignment.up&!buff.incarnation.up|!variable.az_streak|!prev_gcd.1.sunfire)&!(variable.az_potm>=2&spell_targets.moonfire>=2)"
			,"moonfire"
		)


	elseif x.Spec == 2 then
		-- Feral

		x.CP = s.Power("player", Enum.PowerType.ComboPoints)
		x.Energy = s.Power("player", Enum.PowerType.Energy)
		x.MaxEnergy = UnitPowerMax("player", Enum.PowerType.Energy)

		s.UpdateBuff(5215, "Prowl")
		s.UpdateBuff(145152, "Bloodtalons")

		a.Flash("wild_charge_cat")

		-- Executed before combat begins. Accepts non-harmful actions only.
		if not x.InCombat and x.Enemy then
			a.Flash(
				"flask_feral"
				,"food"
				,"augmentation"
				,"regrowth,if=talent.bloodtalons.enabled"
				,"cat_form"
				,"prowl"
				,"potion_feral"
				,"berserk"
			)
			-- It is worth it for almost everyone to maintain thrash
			variable.use_thrash = s.If(azerite.power_of_the_moon.enabled, 1, 2)
			-- Opener TF is delayed if we need to hardcast regrowth later on in the rotation
			-- This happens when Sabertooth, Bloodtalons but not LI is talented
			variable.delayed_tf_opener = s.If(talent.sabertooth.enabled and talent.bloodtalons.enabled and not talent.lunar_inspiration.enabled, 1, 0)
			variable.opener_done = false
			if buff.prowl.down or buff.bloodtalons.down then return end
		end

		a.Flash("skull_bash")

		s.UpdateDoT(1079, "Rip")
		s.UpdateDoT(155722, "Rake")
		s.UpdateDoT(106830, "Thrash")
		s.UpdateDoT(155625, "moonfire_cat")
		s.UpdateBuff(69369, "Predatory Swiftness")
		s.UpdateBuff(135700, "Clearcasting")
		s.UpdateBuff(102543, "incarnation")
		s.UpdateBuff(145152, "Bloodtalons")
		s.UpdateBuff(52610, "Savage Roar")
		s.UpdateBuff(5217, "Tiger's Fury")
		s.UpdateCooldown(5217, "Tiger's Fury")
		s.UpdateBuff(276026, "Iron Jaws")
		s.UpdateBuff(252071, "Jungle Stalker")

		-- Executed every time the actor is available.
		--,"run_action_list,name=opener,if=variable.opener_done=0"
		if not variable.opener_done then
			-- Opener
			-- The opener generally follow the logic of the rest of the apl, but is separated out here for logical clarity
			-- Opening TF will be delayed if we need to hardcast later (which happens with sabertooth & bt but no li)
			-- Otherwise we will open with TF, you can safely cast this from stealth without breaking it.
			variable.opener_done = dot.rip.ticking
			a.Flash(
				"tigers_fury,if=variable.delayed_tf_opener=0"
				-- Always open with rake, consuming stealth and one BT charge (if talented)
				,"rake,if=!ticking|buff.prowl.up"
				-- Lets make sure we end the opener "sequence" when our first rip is ticking
--				,"variable,name=opener_done,value=dot.rip.ticking"
				-- Break out of the action list
--				,"wait,sec=0.001,if=dot.rip.ticking"
				-- If we have LI, and haven't applied it yet use moonfire.
				-- OR, if we only have one BT charge active spam moonfire until we can apply rip with it
				,"moonfire_cat,if=!ticking|buff.bloodtalons.stack=1&combo_points<5"
				-- Given current tuning it is always worth maintaining thrash, so lets apply it here.
				,"thrash,if=!ticking&combo_points<5"
				-- And if none of the above apply, we simply shred until 5
				,"shred,if=combo_points<5"
				-- Regrowth, when we have 5 combo points and bloodtalons and either:
				-- We have Sabertooth and no active bloodtalons stack (via MF spam) even if it requires a hard cast
				-- We have a predatory swiftness proc available (via SR fishing)
				,"regrowth,if=combo_points=5&talent.bloodtalons.enabled&(talent.sabertooth.enabled&buff.bloodtalons.down|buff.predatory_swiftness.up)"
				-- If we ended up hardcasting Regrowth, use TF to re-enter cat form
				,"tigers_fury"
				-- And once we get here at 5 combo_points, just rip and we are up and running
				,"rip,if=combo_points=5"
			)
			return
		end

		if a.Flash(
			-- Single Target
			"cat_form,if=!buff.cat_form.up"
			,"rake,if=buff.prowl.up|buff.shadowmeld.up"
		) then return end

		a.Flash(
			-- Cooldowns
			"dash,if=!buff.cat_form.up"
			,"prowl,if=buff.incarnation.remains<0.5&buff.jungle_stalker.up"
			,"berserk,if=energy>=30&(cooldown.tigers_fury.remains>5|buff.tigers_fury.up)"
			,"tigers_fury,if=energy.deficit>=60"
			,"berserking"
			,"incarnation,if=energy>=30&(cooldown.tigers_fury.remains>15|buff.tigers_fury.up)"
			,"potion,name=battle_potion_of_agility,if=target.time_to_die<65|(time_to_die<180&(buff.berserk.up|buff.incarnation.up))"
--			,"shadowmeld,if=combo_points<5&energy>=action.rake.cost&dot.rake.pmultiplier<2.1&buff.tigers_fury.up&(buff.bloodtalons.up|!talent.bloodtalons.enabled)&(!talent.incarnation.enabled|cooldown.incarnation.remains>18)&!buff.incarnation.up"
--			,"use_items"
		)

		if a.Flash(
			"feral_frenzy,if=combo_points=0"
			,"ferocious_bite,target_if=dot.rip.ticking&dot.rip.remains<3&target.time_to_die>10&(target.health.pct<25|talent.sabertooth.enabled)"
			,"regrowth,if=combo_points=5&buff.predatory_swiftness.up&talent.bloodtalons.enabled&buff.bloodtalons.down&(!buff.incarnation.up|dot.rip.remains<8)"
		) then
			return
		end

		--,"run_action_list,name=st_finishers,if=combo_points>4"
		if x.CP > 4 then
			a.Flash(
				-- ST Finisher
--				"pool_resource,for_next=1"
				"savage_roar,if=buff.savage_roar.down"
--				,"pool_resource,for_next=1"
				,"rip,target_if=!ticking|(remains<=duration*0.3)&(target.health.pct>25&!talent.sabertooth.enabled)|(remains<=duration*0.8&persistent_multiplier>dot.rip.pmultiplier)&target.time_to_die>8"
--				,"pool_resource,for_next=1"
				,"savage_roar,if=buff.savage_roar.remains<12"
--				,"pool_resource,for_next=1"
				,"maim,if=buff.iron_jaws.up"
				,"ferocious_bite,max_energy=1"
			)
			return
		end

		--,"run_action_list,name=st_generators"
		a.Flash(
			-- ST Generators
			"regrowth,if=talent.bloodtalons.enabled&buff.predatory_swiftness.up&buff.bloodtalons.down&combo_points=4&dot.rake.remains<4"
			,"regrowth,if=talent.bloodtalons.enabled&buff.bloodtalons.down&buff.predatory_swiftness.up&talent.lunar_inspiration.enabled&dot.rake.remains<1"
			,"brutal_slash,if=spell_targets.brutal_slash>desired_targets"
--			,"pool_resource,for_next=1"
			,"thrash_cat,if=refreshable&(spell_targets.thrash_cat>2)"
--			,"pool_resource,for_next=1"
			,"rake,target_if=!ticking|(!talent.bloodtalons.enabled&remains<duration*0.3)&target.time_to_die>4"
--			,"pool_resource,for_next=1"
			,"rake,target_if=talent.bloodtalons.enabled&buff.bloodtalons.up&((remains<=7)&persistent_multiplier>dot.rake.pmultiplier*0.85)&target.time_to_die>4"
			-- With LI & BT, we can use moonfire to save BT charges, allowing us to better refresh rake
			,"moonfire_cat,if=buff.bloodtalons.up&buff.predatory_swiftness.down&combo_points<5"
			,"brutal_slash,if=(buff.tigers_fury.up&(raid_event.adds.in>(1+max_charges-charges_fractional)*recharge_time))"
			,"moonfire_cat,target_if=refreshable"
--			,"pool_resource,for_next=1"
			,"thrash_cat,if=refreshable&((variable.use_thrash=2&(!buff.incarnation.up|azerite.wild_fleshrending.enabled))|spell_targets.thrash_cat>1)"
			,"thrash_cat,if=refreshable&variable.use_thrash=1&buff.clearcasting.react&(!buff.incarnation.up|azerite.wild_fleshrending.enabled)"
--			,"pool_resource,for_next=1"
			,"swipe_cat,if=spell_targets.swipe_cat>1"
			,"shred,if=buff.clearcasting.react"
			-- With Power of the Moon traits, Moonfire becomes our primary builder outside of Clearcasting and Incarnation
			,"moonfire_cat,if=azerite.power_of_the_moon.enabled&!buff.incarnation.up"
			,"shred,if=dot.rake.remains>(action.shred.cost+action.rake.cost-energy)/energy.regen|buff.clearcasting.react"
		)

	elseif x.Spec == 3 then
		-- Guardian

		x.Rage = UnitPower("player", Enum.PowerType.Rage)
		a.Flash("wild_charge_bear") -- if we are not in range, use these
		a.Flash("skull_bash") -- interrupt spellcasting
		a.Flash("growl") -- taunt if tanking and not keeping aggro

		-- Executed before combat begins. Accepts non-harmful actions only.
		if not x.InCombat and x.Enemy then
			a.Flash(
				"flask_guardian"
				,"food"
				,"augmentation"
				,"bear_form"
				,"potion_guardian"
			)
		end

		s.UpdateBuff(5487, "Bear Form")
		a.Flash(
			"auto_attack"
			,"potion_guardian"
			,"berserking"
			,"barkskin,if=buff.bear_form.up"
			,"lunar_beam,if=buff.bear_form.up"
			,"bristling_fur,if=buff.bear_form.up"
--			,"use_items"
		)

		s.UpdateDoT(164812, "Moonfire")
		s.UpdateDoT(192090, "Thrash Bear")
		s.UpdateBuff(102558, "Incarnation")
		s.UpdateBuff(213708, "Galactic Guardian")
		a.Flash(
			"maul,if=rage.deficit<10&active_enemies<4"
			,"frenzied_regeneration,if=incoming_damage_5s%health.max>=0.5|health<=health.max*0.4"
			,"ironfur,if=cost=0|(rage>cost&azerite.layered_mane.enabled&active_enemies>2)"
			,"pulverize,target_if=dot.thrash_bear.stack=dot.thrash_bear.max_stacks"
			,"moonfire,target_if=dot.moonfire.refreshable&active_enemies<2"
			,"incarnation"
			,"thrash,if=(buff.incarnation.down&active_enemies>1)|(buff.incarnation.up&active_enemies>4)"
			,"swipe,if=buff.incarnation.down&active_enemies>4"
			,"mangle,if=dot.thrash_bear.ticking"
			,"moonfire,target_if=buff.galactic_guardian.up&active_enemies<2"
			,"thrash"
			,"maul"
			,"swipe"
		)

	end
end)
